TP_DBG_LOCK = 0
TP_DBG_KEYPAD = 1
TP_SER_KEYPAD = 2
TP_VLT_BUZZER = 3
TP_VLT_ERROR = 4

RL_KeypadChip = {{name="ERROR", value="00000000", access=REGACCESS_READ}}

RL_LockChip = { {name="KEYCODE", value="01030904", access=REGACCESS_READ},
				{name="KEYSWAP", value="0F0F0F0F", access=REGACCESS_READ},
				{name="ERROR", value="00000000", access=REGACCESS_READ} }
				
pulseStates = {PULSESTATE_NONE, PULSESTATE_NONE, PULSESTATE_NONE, PULSESTATE_NONE, PULSESTATE_NONE}

errorState = false

API_WidgetSet("DiscreteLED1", "Enabled", true)

function LEVEL_ReadVoltage(pointId)
	if pointId == TP_VLT_BUZZER and errorState == true then
		return 2.5
	end
	return 0
end

function LEVEL_SetPulse(pointId, pulseState)
	pulseStates[pointId+1] = pulseState
end

function LEVEL_ReadDebug(pointId)
	if pointId == TP_DBG_LOCK then
		return RL_LockChip
	elseif pointId == TP_DBG_KEYPAD then
		return RL_KeypadChip
	end
	return nil
end

function LEVEL_WriteDebug(pointId, registerIndex, registerValue)
	-- All of the registers for this level are read only!
end

function LEVEL_WriteSerial(pointId, serialData)
	if pointId == TP_SER_KEYPAD and string.find("123456789",serialData) ~= nil then
		SendButtonData(serialData)
	end
end

function LEVEL_Update(timeDelta)
	if pulseStates[TP_VLT_ERROR+1] == PULSESTATE_HIGH or ( pulseStates[TP_VLT_ERROR+1] ~= PULSESTATE_LOW and errorState == true ) then
		API_WidgetSet("DiscreteLED2", "Enabled", true)
	else
		API_WidgetSet("DiscreteLED2", "Enabled", false)
	end
end

function LEVEL_WidgetEvent(widgetName, eventName)
	if eventName == "pressed" then
		buttonValue = string.sub(widgetName,7)
		SendButtonData(buttonValue)
		API_EmitSerial(TP_SER_KEYPAD, buttonValue)
	end
end

function LEVEL_TimerCallback(timerName)
	if timerName == "errorTimer" then
		errorState = false
		RL_LockChip[3].value = "00000000"
		RL_KeypadChip[1].value = "00000000"
	elseif timerName == "winTimer" then
		API_LevelWin()
	end
end

-- This function is called both by LEVEL_WidgetEvent when pushing a button and LEVEL_WriteSerial when
-- the player sends a serial character to TP_SER_KEYPAD. buttonvalue is a single character string.
function SendButtonData(buttonValue)
	RL_LockChip[2].value = "0" .. buttonValue .. string.sub(RL_LockChip[2].value,1,6)
	if string.sub(RL_LockChip[2].value,8) ~= "F" then
		if RL_LockChip[1].value == RL_LockChip[2].value then
			-- The code matches, so we win!
			API_PlaySound("unlock.wav")
			API_SetTimer("winTimer", 1, false)
			API_WidgetSet("DiscreteLED1", "Enabled", false)
		else
			-- The code doesn't match, so set the error flags.
			API_PlaySound("_error")
			API_SetTimer("errorTimer", 0.25, false)
			errorState = true
			RL_LockChip[3].value = "00000001"
			RL_KeypadChip[1].value = "00000001"
		end
		-- Regardless, reset the swap space.
		RL_LockChip[2].value = "0F0F0F0F"
	end
end
